@@ -1,13 +1,13 @@
-from datetime import datetime
-import time
-
 from django.conf import settings
 from django.http import QueryDict
 from django.core.exceptions import ImproperlyConfigured
 
 from fandjango.views import authorize_application, authorization_denied
 from fandjango.models import Facebook, User, OAuthToken
-from fandjango.settings import FACEBOOK_APPLICATION_SECRET_KEY, FACEBOOK_APPLICATION_ID, DISABLED_PATHS, ENABLED_PATHS
+from fandjango.settings import (
+    FACEBOOK_APPLICATION_SECRET_KEY, FACEBOOK_APPLICATION_ID,
+    FANDJANGO_CACHE_SIGNED_REQUEST, DISABLED_PATHS, ENABLED_PATHS
+)
 from fandjango.utils import (
     is_disabled_path, is_enabled_path,
     authorization_denied_view, get_post_authorization_redirect_url
@@ -15,6 +15,15 @@ from fandjango.utils import (
 
 from facepy import SignedRequest, GraphAPI
 
+try:
+    from django.utils.timezone import now
+except ImportError:
+    from datetime import datetime
+    def now():
+        return datetime.now()
+
+from dateutil.tz import tzlocal
+
 class FacebookMiddleware():
     """Middleware for Facebook applications."""
 
@@ -22,7 +31,10 @@ class FacebookMiddleware():
         """Process the signed request."""
 
         if ENABLED_PATHS and DISABLED_PATHS:
-            raise ImproperlyConfigured('You may configure either FANDJANGO_ENABLED_PATHS or FANDJANGO_DISABLED_PATHS, but not both.')
+            raise ImproperlyConfigured(
+                'You may configure either FANDJANGO_ENABLED_PATHS '
+                'or FANDJANGO_DISABLED_PATHS, but not both.'
+            )
 
         if DISABLED_PATHS and is_disabled_path(request.path):
             return
@@ -30,7 +42,7 @@ class FacebookMiddleware():
         if ENABLED_PATHS and not is_enabled_path(request.path):
             return
 
-        # An error occured during authorization...        
+        # An error occured during authorization...
         if 'error' in request.GET:
             error = request.GET['error']
 
@@ -78,8 +90,8 @@ class FacebookMiddleware():
                 except User.DoesNotExist:
                     oauth_token = OAuthToken.objects.create(
                         token = request.facebook.signed_request.user.oauth_token.token,
-                        issued_at = request.facebook.signed_request.user.oauth_token.issued_at,
-                        expires_at = request.facebook.signed_request.user.oauth_token.expires_at
+                        issued_at = request.facebook.signed_request.user.oauth_token.issued_at.replace(tzinfo=tzlocal()),
+                        expires_at = request.facebook.signed_request.user.oauth_token.expires_at.replace(tzinfo=tzlocal())
                     )
 
                     user = User.objects.create(
@@ -91,14 +103,16 @@ class FacebookMiddleware():
 
                 # Update the user's details and OAuth token
                 else:
-                    user.last_seen_at = datetime.now()
-                    user.authorized = True
+                    user.last_seen_at = now()
 
-                    if request.facebook.signed_request.user.oauth_token:
-                        user.oauth_token.token = request.facebook.signed_request.user.oauth_token.token
-                        user.oauth_token.issued_at = request.facebook.signed_request.user.oauth_token.issued_at
-                        user.oauth_token.expires_at = request.facebook.signed_request.user.oauth_token.expires_at
-                        user.oauth_token.save()
+                    if 'signed_request' in request.REQUEST:
+                        user.authorized = True
+
+                        if request.facebook.signed_request.user.oauth_token:
+                            user.oauth_token.token = request.facebook.signed_request.user.oauth_token.token
+                            user.oauth_token.issued_at = request.facebook.signed_request.user.oauth_token.issued_at.replace(tzinfo=tzlocal())
+                            user.oauth_token.expires_at = request.facebook.signed_request.user.oauth_token.expires_at.replace(tzinfo=tzlocal())
+                            user.oauth_token.save()
 
                     user.save()
 
@@ -126,7 +140,9 @@ class FacebookMiddleware():
         browsers it is considered by IE before accepting third-party cookies (ie. cookies set by
         documents in iframes). If they are not set correctly, IE will not set these cookies.
         """
-        if 'signed_request' in request.REQUEST:
-            response.set_cookie('signed_request', request.REQUEST['signed_request'])
-        response['P3P'] = 'CP="IDC CURa ADMa OUR IND PHY ONL COM STA"'
+        if FANDJANGO_CACHE_SIGNED_REQUEST:
+            if request.facebook:
+                response.set_cookie('signed_request', request.facebook.signed_request.generate())
+            response['P3P'] = 'CP="IDC CURa ADMa OUR IND PHY ONL COM STA"'
         return response
+
